Package: updog
Title: Flexible Genotyping for Polyploids
Version: 2.1.7
Authors@R: person("David", 
                  "Gerard",
                  email = "gerard.1787@gmail.com", 
                  role = c("aut", "cre"), 
                  comment = c(ORCID = "0000-0001-9450-5023"))
Description: Implements empirical Bayes approaches to genotype
       polyploids from next generation sequencing data while
       accounting for allele bias, overdispersion, and sequencing
       error. The main functions are flexdog() and multidog(), 
       which allow the specification
       of many different genotype distributions. Also provided are functions to
       simulate genotypes, rgeno(), and read-counts, rflexdog(), as well as
       functions to calculate oracle genotyping error rates, oracle_mis(), and
       correlation with the true genotypes, oracle_cor(). These latter two
       functions are useful for read depth calculations. Run
       browseVignettes(package = "updog") in R for example usage. See
       Gerard et al. (2018) <doi:10.1534/genetics.118.301468> and
       Gerard and Ferrao (2020) <doi:10.1093/bioinformatics/btz852> for details 
       on the implemented methods.
Depends: R (>= 3.4.0)
License: GPL (>= 3)
BugReports: https://github.com/dcgerard/updog/issues
URL: https://dcgerard.github.io/updog/
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp (>= 0.12.16), RcppArmadillo, assertthat, ggplot2,
        reshape2, foreach, future, doFuture, doRNG, methods, iterators,
        parallelly
Suggests: covr, testthat (>= 3.0.0), knitr, rmarkdown,
        VariantAnnotation, GenomicRanges, S4Vectors, IRanges
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-03-05 16:05:12 UTC; dgerard
Author: David Gerard [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9450-5023>)
Maintainer: David Gerard <gerard.1787@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-17 17:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 08:05:50 UTC; unix
Archs: updog.so.dSYM
