# Generated by roxygen2: do not edit by hand

S3method(plot,flexdog)
S3method(plot,multidog)
export(dbetabinom)
export(filter_snp)
export(flexdog)
export(flexdog_full)
export(format_multidog)
export(get_q_array)
export(is.flexdog)
export(is.multidog)
export(log_sum_exp)
export(log_sum_exp_2)
export(multidog)
export(oracle_cor)
export(oracle_cor_from_joint)
export(oracle_joint)
export(oracle_mis)
export(oracle_mis_from_joint)
export(oracle_mis_vec)
export(oracle_mis_vec_from_joint)
export(oracle_plot)
export(pbetabinom)
export(plot_geno)
export(qbetabinom)
export(rbetabinom)
export(rflexdog)
export(rgeno)
export(wem)
import(doFuture)
importFrom(Rcpp,sourceCpp)
importFrom(RcppArmadillo,armadillo_version)
importFrom(assertthat,assert_that)
importFrom(doRNG,"%dorng%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,ggplot)
importFrom(methods,as)
importFrom(parallelly,availableCores)
useDynLib(updog)
